"""Tests for validation exercises"""
import unittest


from validation import (
    has_word,
    is_email,
    is_phone_number,
    get_4_letter_words,
    get_email,
)


class HasWordTests(unittest.TestCase):

    """Tests for has_word."""

    def test_just_the_word(self):
        sentence = "help"
        self.assertTrue(has_word("help", sentence))

    def test_different_capitalization(self):
        sentence = "Help"
        self.assertTrue(has_word("help", sentence))

    def test_whole_word_mid_sentence(self):
        sentence = "She was a big help when I learned French"
        self.assertTrue(has_word("help", sentence))

    def test_punctuation(self):
        sentence = "Help! I need somebody."
        self.assertTrue(has_word("help", sentence))

    def test_word_prefix(self):
        sentence = "She helped me learn French"
        self.assertFalse(has_word("help", sentence))

    def test_word_suffix(self):
        sentence = "feel"
        self.assertFalse(has_word("eel", sentence))


class IsEmailTests(unittest.TestCase):

    """Tests for is_email."""

    def test_digits(self):
        self.assertTrue(is_email("123@example.com"))

    def test_plus(self):
        self.assertTrue(is_email("myname+spam@gmail.com"))

    def test_subdomain(self):
        self.assertTrue(is_email("info@help.example.com"))

    # def test_subdomain(self):
    #     self.assertTrue(is_email("info@help.example.com"))

    def test_long_tld(self):
        self.assertTrue(is_email("me@something.technology"))

    def test_short_email(self):
        self.assertTrue(is_email("a@b.co"))

    def test_symbols(self):
        self.assertTrue(is_email("my_name+with%symbols@b.com"))

    def test_capitals(self):
        self.assertTrue(is_email("A@BCD.COM"))

    def test_single_letter_tld(self):
        self.assertFalse(is_email("123@example.c"))

    def test_no_name(self):
        self.assertFalse(is_email("@b.co"))

    def test_no_domain(self):
        self.assertFalse(is_email("a@.co"))

    def test_no_period(self):
        self.assertFalse(is_email("a@abcd"))


class IsPhoneNumberTests(unittest.TestCase):

    """Tests for is_phone_number."""

    def test_no_punctuation(self):
        self.assertTrue(is_phone_number("8885557777"))

    def test_dashes(self):
        self.assertTrue(is_phone_number('202-762-1401'))

    def test_parenthesis_without_space(self):
        self.assertTrue(is_phone_number('(202)762-1401'))

    def test_parenthesis_with_space(self):
        self.assertTrue(is_phone_number('(202) 762-1401'))

    def test_periods(self):
        self.assertTrue(is_phone_number('202 . 762 . 1401'))

    def test_incorrect_dash_placement(self):
        self.assertFalse(is_phone_number('20-2762-1401'))

    def test_letters(self):
        self.assertFalse(is_phone_number('123-456-78A9'))

    def test_too_short(self):
        self.assertFalse(is_phone_number('123456789'))

    def test_too_long(self):
        self.assertFalse(is_phone_number('12345678901'))

    def test_incorrect_period_placement(self):
        self.assertFalse(is_phone_number('123 . 4567 . 890'))


class Get4LetterWordsTests(unittest.TestCase):

    """Tests for get_4_letter_words."""

    def test_single_word(self):
        self.assertEqual(get_4_letter_words("hiya"), ["hiya"])

    def test_three_letter_word(self):
        self.assertEqual(get_4_letter_words("hey"), [])

    def test_five_letter_word(self):
        self.assertEqual(get_4_letter_words("hello"), [])

    def test_multiple_words(self):
        sentence = "four letter word"
        words = ["four", "word"]
        self.assertEqual(get_4_letter_words(sentence), words)

    def test_capitalization(self):
        sentence = "Hi my name is Trey"
        words = ["name", "Trey"]
        self.assertEqual(get_4_letter_words(sentence), words)

    def test_punctuation(self):
        sentence = "What?!  What have you done?"
        words = ["What", "What", "have", "done"]
        self.assertEqual(get_4_letter_words(sentence), words)


class GetEmailTests(unittest.TestCase):

    """Tests for get_email."""

    def test_ending_in_email(self):
        sentence = "String with email 123@example.com"
        self.assertEqual(get_email(sentence), "123@example.com")

    def test_only_email(self):
        self.assertEqual(get_email("hello@domain.co"), "hello@domain.co")

    def test_punctuation_after_email(self):
        sentence = "Email is 123@example.com."
        self.assertEqual(get_email(sentence), "123@example.com")

    def test_two_emails(self):
        sentence = "abc@def.com ghi@jkl.com"
        self.assertEqual(get_email(sentence), "abc@def.com")

    def test_invalid_email(self):
        sentence = "a@b.c"
        self.assertIsNone(get_email(sentence))


if __name__ == "__main__":
    raise SystemExit("No CLI for this file.  Run test.py instead.")
